#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Parameters.h"
#include "cctk_Functions.h"
#include "cctk_Faces.h"
#include "util_Table.h"
#include "Symmetry.h"

void ZelmaniAnalysis_SelectBoundConds(CCTK_ARGUMENTS)
{
  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;

  CCTK_INT ierr = 0;

  ierr = Boundary_SelectVarForBC (cctkGH, CCTK_ALL_FACES, 1, -1, 
				  "ZelmaniAnalysis::gravitational_mass_local", "none");
  if (ierr<0) CCTK_WARN (CCTK_WARN_ABORT, 
			 "Failed to select boundary condition for ZelmaniAnalysis::gravitational_mass_local");


  ierr = Boundary_SelectVarForBC (cctkGH, CCTK_ALL_FACES, 1, -1, 
				  "ZelmaniAnalysis::gravitational_mass_local_in1e12", "none");
  if (ierr<0) CCTK_WARN (CCTK_WARN_ABORT, 
			 "Failed to select boundary condition for ZelmaniAnalysis::gravitational_mass_local_in1e12");


  ierr = Boundary_SelectVarForBC (cctkGH, CCTK_ALL_FACES, 1, -1, 
				  "ZelmaniAnalysis::proper_mass_local", "none");
  if (ierr<0) CCTK_WARN (CCTK_WARN_ABORT, 
			 "Failed to select boundary condition for ZelmaniAnalysis::proper_mass_local");

  ierr = Boundary_SelectVarForBC (cctkGH, CCTK_ALL_FACES, 1, -1, 
				  "ZelmaniAnalysis::baryonic_mass_local_in1e12", "none");
  if (ierr<0) CCTK_WARN (CCTK_WARN_ABORT, 
			 "Failed to select boundary condition for ZelmaniAnalysis::baryonic_mass_local_in1e12");

  ierr = Boundary_SelectVarForBC (cctkGH, CCTK_ALL_FACES, 1, -1, 
				  "ZelmaniAnalysis::total_kinetic_energy_local", "none");
  if (ierr<0) CCTK_WARN (CCTK_WARN_ABORT, 
			 "Failed to select boundary condition for ZelmaniAnalysis::total_kinetic_energy_local");
  
  ierr = Boundary_SelectVarForBC (cctkGH, CCTK_ALL_FACES, 1, -1, 
				  "ZelmaniAnalysis::kinetic_energy_local", "none");
  if (ierr<0) CCTK_WARN (CCTK_WARN_ABORT, 
			 "Failed to select boundary condition for ZelmaniAnalysis::kinetic_energy_local");
  
  ierr = Boundary_SelectVarForBC (cctkGH, CCTK_ALL_FACES, 1, -1, 
				  "ZelmaniAnalysis::angular_momentum_local", "none");
  if (ierr<0) CCTK_WARN (CCTK_WARN_ABORT, 
			 "Failed to select boundary condition for ZelmaniAnalysis::angular_momentum_local");

  return;
}

void ZelmaniAnalysis_RegisterSymmetries(CCTK_ARGUMENTS)
{
  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;


  /* array holding symmetry definitions */
  CCTK_INT sym[3];

  sym[0] = 1;
  sym[1] = 1;
  sym[2] = 1;
  SetCartSymVN(cctkGH, sym, "ZelmaniAnalysis::gravitational_mass_local");
  SetCartSymVN(cctkGH, sym, "ZelmaniAnalysis::gravitational_mass_local_in1e12");
  SetCartSymVN(cctkGH, sym, "ZelmaniAnalysis::proper_mass_local");
  SetCartSymVN(cctkGH, sym, "ZelmaniAnalysis::baryonic_mass_local_in1e12");
  SetCartSymVN(cctkGH, sym, "ZelmaniAnalysis::total_kinetic_energy_local");
  SetCartSymVN(cctkGH, sym, "ZelmaniAnalysis::kinetic_energy_local");


  sym[0] = 1;
  sym[1] = 1;
  sym[2] = -1;
  SetCartSymVN(cctkGH, sym, "ZelmaniAnalysis::angular_momentum_local");



}
